import { createContext, useRef, useCallback, useMemo, useContext } from 'react';
import { KEY_IS_NOT_STRING_ERROR, DEFAULT_MAX_CONCURRENT_REQUESTS } from './constants.js';
import DataLoader from './dataloader.js';
import { jsx } from 'react/jsx-runtime';

const DataLoaderContext = createContext(undefined);
const DataLoaderProvider = _ref => {
  let {
    children,
    cacheKeyPrefix = '',
    onError,
    maxConcurrentRequests = DEFAULT_MAX_CONCURRENT_REQUESTS
  } = _ref;
  const requestsRef = useRef({});
  const computeKey = useCallback(key => `${cacheKeyPrefix ? `${cacheKeyPrefix}-` : ''}${key}`, [cacheKeyPrefix]);
  const getRequest = useCallback(key => requestsRef.current[computeKey(key)], [computeKey]);
  const addRequest = useCallback((key, args) => {
    if (DataLoader.maxConcurrent !== maxConcurrentRequests) {
      DataLoader.maxConcurrent = maxConcurrentRequests;
    }
    if (key && typeof key === 'string') {
      const newRequest = new DataLoader({
        ...args,
        key: computeKey(key)
      });
      requestsRef.current[newRequest.key] = newRequest;
      return newRequest;
    }
    throw new Error(KEY_IS_NOT_STRING_ERROR);
  }, [computeKey, maxConcurrentRequests]);
  const getOrAddRequest = useCallback((key, args) => {
    const requestFound = getRequest(key);
    if (!requestFound) {
      return addRequest(key, args);
    }
    return requestFound;
  }, [addRequest, getRequest]);
  const clearCachedData = useCallback(key => {
    if (key && typeof key === 'string') {
      getRequest(key)?.clearData();
    } else throw new Error(KEY_IS_NOT_STRING_ERROR);
  }, [getRequest]);
  const clearAllCachedData = useCallback(() => {
    Object.values(requestsRef.current).forEach(request => {
      request.clearData();
    });
  }, []);
  const reload = useCallback(async key => {
    if (key && typeof key === 'string') {
      await getRequest(key)?.load(true);
    } else throw new Error(KEY_IS_NOT_STRING_ERROR);
  }, [getRequest]);
  const reloadAll = useCallback(async () => {
    await Promise.all(Object.values(requestsRef.current).map(request => request.load(true)));
  }, []);
  const getCachedData = useCallback(key => {
    if (key) {
      return getRequest(key)?.getData();
    }
    return Object.values(requestsRef.current).reduce((acc, request) => ({
      ...acc,
      [request.key]: request.getData()
    }), {});
  }, [getRequest]);
  const getReloads = useCallback(key => {
    if (key) {
      return getRequest(key) ? () => getRequest(key)?.load(true) : undefined;
    }
    return Object.entries(requestsRef.current).reduce((acc, _ref2) => {
      let [requestKey, {
        load
      }] = _ref2;
      return {
        ...acc,
        [requestKey]: () => load(true)
      };
    }, {});
  }, [getRequest]);
  const value = useMemo(() => ({
    addRequest,
    cacheKeyPrefix,
    clearAllCachedData,
    clearCachedData,
    getCachedData,
    getOrAddRequest,
    getReloads,
    getRequest,
    onError,
    reload,
    reloadAll
  }), [addRequest, cacheKeyPrefix, clearAllCachedData, clearCachedData, getCachedData, getOrAddRequest, getRequest, getReloads, onError, reload, reloadAll]);
  return jsx(DataLoaderContext.Provider, {
    value: value,
    children: children
  });
};
const useDataLoaderContext = () => useContext(DataLoaderContext);
var DataLoaderProvider$1 = DataLoaderProvider;

export { DataLoaderContext, DataLoaderProvider$1 as default, useDataLoaderContext };
